@@ -5,7 +5,7 @@ use Mango::BSON 'bson_doc';
 use Mojo::IOLoop;
 
 has [qw(batch_size limit skip)] => 0;
-has [qw(collection id sort)];
+has [qw(collection hint id snapshot sort tailable)];
 has [qw(fields query)] => sub { {} };
 
 sub all {
@@ -20,6 +20,34 @@ sub all {
   return \@all;
 }
 
+sub build_query {
+  my ($self, $explain) = @_;
+
+  my $query    = $self->query;
+  my $sort     = $self->sort;
+  my $hint     = $self->hint;
+  my $snapshot = $self->snapshot;
+
+  return $query unless $snapshot || $hint || $sort || $explain;
+
+  $query = {'$query' => $query};
+  $query->{'$explain'}  = 1     if $explain;
+  $query->{'$orderby'}  = $sort if $sort;
+  $query->{'$hint'}     = $hint if $hint;
+  $query->{'$snapshot'} = 1     if $snapshot;
+
+  return $query;
+}
+
+sub clone {
+  my $self  = shift;
+  my $clone = $self->new;
+  $clone->$_($self->$_)
+    for qw(batch_size collection fields hint limit query skip snapshot),
+    qw(sort tailable);
+  return $clone;
+}
+
 sub count {
   my $self = shift;
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
@@ -27,7 +55,7 @@ sub count {
   my $collection = $self->collection;
   my $count      = bson_doc
     count => $collection->name,
-    query => $self->_query,
+    query => $self->build_query,
     skip  => $self->skip,
     limit => $self->limit;
 
@@ -44,6 +72,17 @@ sub count {
   return $doc ? $doc->{n} : 0;
 }
 
+sub explain {
+  my ($self, $cb) = @_;
+
+  # Non-blocking
+  my $clone = $self->clone->query($self->build_query(1))->sort(undef);
+  return $clone->next(sub { shift; $self->$cb(@_) }) if $cb;
+
+  # Blocking
+  return $clone->next;
+}
+
 sub next {
   my ($self, $cb) = @_;
   return exists $self->{results} ? $self->_continue($cb) : $self->_start($cb);
@@ -57,11 +96,11 @@ sub rewind {
   $self->id(undef);
 
   # Non-blocking
-  return $self->collection->db->mango->kill_cursors($id => sub { $self->$cb })
-    if $cb;
+  my $mango = $self->collection->db->mango;
+  return $mango->kill_cursors($id => sub { shift; $self->$cb(@_) }) if $cb;
 
   # Blocking
-  $self->collection->db->mango->kill_cursors($id);
+  $mango->kill_cursors($id);
 }
 
 sub _collect {
@@ -74,23 +113,20 @@ sub _collect {
 sub _continue {
   my ($self, $cb) = @_;
 
-  # Non-blocking
   my $collection = $self->collection;
   my $name       = $collection->full_name;
+  my $mango      = $collection->db->mango;
+
+  # Non-blocking
   if ($cb) {
     return $self->_defer($cb, undef, $self->_dequeue) if $self->_enough;
-    return $collection->db->mango->get_more(
-      ($name, $self->_max, $self->id) => sub {
-        my ($mango, $err, $reply) = @_;
-        $self->$cb($err, $self->_enqueue($reply));
-      }
-    );
+    return $mango->get_more(($name, $self->_max, $self->id) =>
+        sub { shift; $self->$cb(shift, $self->_enqueue(shift)) });
   }
 
   # Blocking
   return $self->_dequeue if $self->_enough;
-  return $self->_enqueue(
-    $collection->db->mango->get_more($name, $self->_max, $self->id));
+  return $self->_enqueue($mango->get_more($name, $self->_max, $self->id));
 }
 
 sub _defer {
@@ -118,7 +154,7 @@ sub _enqueue {
   my ($self, $reply) = @_;
   return unless $reply;
   push @{$self->{results} ||= []}, @{$reply->{docs}};
-  return $self->_dequeue;
+  return $self->id($reply->{cursor})->_dequeue;
 }
 
 sub _max {
@@ -129,29 +165,20 @@ sub _max {
   return $size > $limit ? $limit : $size;
 }
 
-sub _query {
-  my $self  = shift;
-  my $query = $self->query;
-  return $query unless my $sort = $self->sort;
-  return {'$query' => $query, '$orderby' => $sort};
-}
-
 sub _start {
   my ($self, $cb) = @_;
 
   my $collection = $self->collection;
   my $name       = $collection->full_name;
-  my @args
-    = ($name, {}, $self->skip, $self->_max, $self->_query, $self->fields);
+  my $flags = $self->tailable ? {tailable_cursor => 1, await_data => 1} : {};
+  my @args  = (
+    $name, $flags, $self->skip, $self->_max, $self->build_query, $self->fields
+  );
 
   # Non-blocking
   return $collection->db->mango->query(
-    @args => sub {
-      my ($mango, $err, $reply) = @_;
-      $self->id($reply->{cursor}) if $reply;
-      $self->$cb($err, $self->_enqueue($reply));
-    }
-  ) if $cb;
+    @args => sub { shift; $self->$cb(shift, $self->_enqueue(shift)) })
+    if $cb;
 
   # Blocking
   my $reply = $collection->db->mango->query(@args);
@@ -185,7 +212,7 @@ L<Mango::Cursor> implements the following attributes.
   my $size = $cursor->batch_size;
   $cursor  = $cursor->batch_size(10);
 
-Batch size, defaults to C<0>.
+Number of documents to fetch in one batch, defaults to C<0>.
 
 =head2 collection
 
@@ -194,6 +221,20 @@ Batch size, defaults to C<0>.
 
 L<Mango::Collection> object this cursor belongs to.
 
+=head2 fields
+
+  my $fields = $cursor->fields;
+  $cursor    = $cursor->fields({foo => 1});
+
+Select fields from documents.
+
+=head2 hint
+
+  my $hint = $cursor->hint;
+  $cursor  = $cursor->hint({foo => 1});
+
+Force a specific index to be used.
+
 =head2 id
 
   my $id  = $cursor->id;
@@ -206,35 +247,42 @@ Cursor id.
   my $limit = $cursor->limit;
   $cursor   = $cursor->limit(10);
 
-Limit, defaults to C<0>.
-
-=head2 fields
-
-  my $fields = $cursor->fields;
-  $cursor    = $cursor->fields({foo => 1});
-
-Fields.
+Limit the number of documents, defaults to C<0>.
 
 =head2 query
 
   my $query = $cursor->query;
   $cursor   = $cursor->query({foo => 'bar'});
 
-Query.
+Original query.
 
 =head2 skip
 
   my $skip = $cursor->skip;
   $cursor  = $cursor->skip(5);
 
-Documents to skip, defaults to C<0>.
+Number of documents to skip, defaults to C<0>.
+
+=head2 snapshot
+
+  my $snapshot = $cursor->snapshot;
+  $cursor      = $cursor->snapshot(1);
+
+Use snapshot mode.
 
 =head2 sort
 
   my $sort = $cursor->sort;
   $cursor  = $cursor->sort({foo => 1});
 
-Sort.
+Sort documents.
+
+=head2 tailable
+
+  my $tailable = $cursor->tailable;
+  $cursor      = $cursor->tailable(1);
+
+Tailable cursor.
 
 =head1 METHODS
 
@@ -254,6 +302,19 @@ non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
+=head2 build_query
+
+  my $query = $cursor->build_query;
+  my $query = $cursor->build_query($explain);
+
+Generate final query with cursor attributes.
+
+=head2 clone
+
+  my $clone = $cursor->clone;
+
+Clone cursor.
+
 =head2 count
 
   my $count = $cursor->count;
@@ -267,6 +328,19 @@ callback to perform operation non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
+=head2 explain
+
+  my $doc = $cursor->explain;
+
+Provide information on the query plan. You can also append a callback to
+perform operation non-blocking.
+
+  $cursor->explain(sub {
+    my ($cursor, $err, $doc) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 next
 
   my $doc  = $cursor->next;
@@ -288,7 +362,7 @@ Rewind cursor. You can also append a callback to perform operation
 non-blocking.
 
   $cursor->rewind(sub {
-    my $cursor = shift;
+    my ($cursor, $err) = @_;
     ...
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
